@@ -24,9 +24,11 @@ module Heist
   , initHeistWithCacheTag
   , defaultInterpretedSplices
   , defaultLoadTimeSplices
+  , emptyHeistConfig
 
   -- * Core Heist data types
-  , HeistConfig(..)
+  , SpliceConfig
+  , HeistConfig
   , TemplateRepo
   , TemplateLocation
   , Template
@@ -37,12 +39,29 @@ module Heist
   , RuntimeSplice
   , Chunk
   , HeistState
+  , HeistT
+
+  -- * Lenses (can be used with lens or lens-family)
+  , scInterpretedSplices
+  , scLoadTimeSplices
+  , scCompiledSplices
+  , scAttributeSplices
+  , scTemplateLocations
+  , hcSpliceConfig
+  , hcNamespace
+  , hcErrorNotBound
+  , hcInterpretedSplices
+  , hcLoadTimeSplices
+  , hcCompiledSplices
+  , hcAttributeSplices
+  , hcTemplateLocations
+
+  -- * HeistT functions
   , templateNames
   , compiledTemplateNames
   , hasTemplate
   , spliceNames
   , compiledSpliceNames
-  , HeistT
   , evalHeistT
   , getParamNode
   , getContext
@@ -56,74 +75,37 @@ module Heist
   , localHS
   , getDoc
   , getXMLDoc
+  , tellSpliceError
   , orError
-
-  -- * Splice API
   , module Heist.SpliceAPI
   ) where
 
+
+------------------------------------------------------------------------------
 import           Control.Error
 import           Control.Exception (SomeException)
 import           Control.Monad.CatchIO
 import           Control.Monad.State
-import           Data.ByteString (ByteString)
-import qualified Data.ByteString as B
-import qualified Data.Foldable as F
-import qualified Data.HeterogeneousEnvironment   as HE
-import           Data.HashMap.Strict (HashMap)
-import qualified Data.HashMap.Strict as Map
+import           Data.ByteString               (ByteString)
+import qualified Data.ByteString               as B
+import qualified Data.Foldable                 as F
+import           Data.HashMap.Strict           (HashMap)
+import qualified Data.HashMap.Strict           as Map
+import qualified Data.HeterogeneousEnvironment as HE
+import           Data.Map.Syntax
 import           Data.Monoid
+import           Data.Text                     (Text)
+import qualified Data.Text                     as T
 import           System.Directory.Tree
-import qualified Text.XmlHtml                    as X
-
+import qualified Text.XmlHtml                  as X
+------------------------------------------------------------------------------
 import           Heist.Common
-import qualified Heist.Compiled.Internal as C
-import qualified Heist.Interpreted.Internal as I
+import qualified Heist.Compiled.Internal       as C
+import qualified Heist.Interpreted.Internal    as I
 import           Heist.SpliceAPI
 import           Heist.Splices
-import           Heist.Types
-
-
-type TemplateRepo = HashMap TPath DocumentFile
-
-
+import           Heist.Internal.Types
 ------------------------------------------------------------------------------
--- | An IO action for getting a template repo from this location.  By not just
--- using a directory path here, we support templates loaded from a database,
--- retrieved from the network, or anything else you can think of.
-type TemplateLocation = EitherT [String] IO TemplateRepo
-
-
-data HeistConfig m = HeistConfig
-    { hcInterpretedSplices :: Splices (I.Splice m)
-        -- ^ Interpreted splices are the splices that Heist has always had.  They
-        -- return a list of nodes and are processed at runtime.
-    , hcLoadTimeSplices    :: Splices (I.Splice IO)
-        -- ^ Load time splices are like interpreted splices because they return a
-        -- list of nodes.  But they are like compiled splices because they are
-        -- processed once at load time.  All of Heist's built-in splices should be
-        -- used as load time splices.
-    , hcCompiledSplices    :: Splices (C.Splice m)
-        -- ^ Compiled splices return a DList of Chunks and are processed at load
-        -- time to generate a runtime monad action that will be used to render the
-        -- template.
-    , hcAttributeSplices   :: Splices (AttrSplice m)
-        -- ^ Attribute splices are bound to attribute names and return a list of
-        -- attributes.
-    , hcTemplateLocations  :: [TemplateLocation]
-        -- ^ A list of all the locations that Heist should get its templates
-        -- from.  
-    }
-
-
-instance Monoid (HeistConfig m) where
-    mempty = HeistConfig (put mempty) (put mempty) (put mempty) (put mempty) mempty
-    mappend (HeistConfig a b c d e) (HeistConfig a' b' c' d' e') =
-        HeistConfig (unionWithS const a a')
-                    (unionWithS const b b')
-                    (unionWithS const c c')
-                    (unionWithS const d d')
-                    (e `mappend` e')
 
 
 ------------------------------------------------------------------------------
@@ -133,18 +115,20 @@ instance Monoid (HeistConfig m) where
 -- old content tag, which has now been moved to two separate tags called
 -- apply-content and bind-content.
 defaultLoadTimeSplices :: MonadIO m => Splices (I.Splice m)
-defaultLoadTimeSplices =
+defaultLoadTimeSplices = do
     -- To be removed in later versions
-    insertS "content" deprecatedContentCheck defaultInterpretedSplices
-    
+    defaultInterpretedSplices
+    "content" #! deprecatedContentCheck
+
 
 
 ------------------------------------------------------------------------------
 -- | The built-in set of static splices.  All the splices that used to be
 -- enabled by default are included here.  To get the normal Heist behavior you
--- should include these in the hcLoadTimeSplices list in your HeistConfig.  If
--- you are using interpreted splice mode, then you might also want to bind the
--- 'deprecatedContentCheck' splice to the content tag as a load time splice.
+-- should include these in the scLoadTimeSplices list in your SpliceConfig.
+-- If you are using interpreted splice mode, then you might also want to bind
+-- the 'deprecatedContentCheck' splice to the content tag as a load time
+-- splice.
 defaultInterpretedSplices :: MonadIO m => Splices (I.Splice m)
 defaultInterpretedSplices = do
     applyTag ## applyImpl
@@ -154,6 +138,13 @@ defaultInterpretedSplices = do
 
 
 
+------------------------------------------------------------------------------
+-- | An empty HeistConfig that uses the \"h\" namespace with error checking
+-- turned on.
+emptyHeistConfig :: HeistConfig m
+emptyHeistConfig = HeistConfig mempty "h" True
+
+
 allErrors :: [Either String (TPath, v)]
           -> EitherT [String] IO (HashMap TPath v)
 allErrors tlist =
@@ -210,7 +201,7 @@ addTemplatePathPrefix dir ts
 -- | Creates an empty HeistState.
 emptyHS :: HE.KeyGen -> HeistState m
 emptyHS kg = HeistState Map.empty Map.empty Map.empty Map.empty Map.empty
-                        True [] 0 [] Nothing kg False Html
+                        True [] 0 [] Nothing kg False Html "" [] False
 
 
 ------------------------------------------------------------------------------
@@ -235,24 +226,45 @@ initHeist :: Monad n
           -> EitherT [String] IO (HeistState n)
 initHeist hc = do
     keyGen <- lift HE.newKeyGen
-    repos <- sequence $ hcTemplateLocations hc
+    repos <- sequence $ _scTemplateLocations $ _hcSpliceConfig hc
     initHeist' keyGen hc (Map.unions repos)
 
 
+------------------------------------------------------------------------------
+mkSplicePrefix :: Text -> Text
+mkSplicePrefix ns
+  | T.null ns = ""
+  | otherwise = ns `mappend` ":"
+
+
+------------------------------------------------------------------------------
 initHeist' :: Monad n
            => HE.KeyGen
            -> HeistConfig n
            -> TemplateRepo
            -> EitherT [String] IO (HeistState n)
-initHeist' keyGen (HeistConfig i lt c a _) repo = do
+initHeist' keyGen (HeistConfig sc ns enn) repo = do
     let empty = emptyHS keyGen
-    tmap <- preproc keyGen lt repo
-    let hs1 = empty { _spliceMap = Map.fromList $ splicesToList i
+    let (SpliceConfig i lt c a _) = sc
+    tmap <- preproc keyGen lt repo ns
+    let prefix = mkSplicePrefix ns
+    is <- runHashMap $ mapK (prefix<>) i
+    cs <- runHashMap $ mapK (prefix<>) c
+    as <- runHashMap $ mapK (prefix<>) a
+    let hs1 = empty { _spliceMap = is
                     , _templateMap = tmap
-                    , _compiledSpliceMap = Map.fromList $ splicesToList c
-                    , _attrSpliceMap = Map.fromList $ splicesToList a
+                    , _compiledSpliceMap = cs
+                    , _attrSpliceMap = as
+                    , _splicePrefix = prefix
+                    , _errorNotBound = enn
                     }
-    lift $ C.compileTemplates hs1
+    EitherT $ C.compileTemplates hs1
+--    liftIO $ when (not $ null $ _spliceErrors hs2) $ do
+--        putStrLn "Finished compiling with errors..."
+--        mapM_ T.putStrLn $ _spliceErrors hs2
+--    case _spliceErrors hs2 of
+--      [] -> return hs2
+--      es -> left $ map T.unpack es
 
 
 ------------------------------------------------------------------------------
@@ -260,11 +272,14 @@ initHeist' keyGen (HeistConfig i lt c a _) repo = do
 preproc :: HE.KeyGen
         -> Splices (I.Splice IO)
         -> TemplateRepo
+        -> Text
         -> EitherT [String] IO TemplateRepo
-preproc keyGen splices templates = do
-    let hs = (emptyHS keyGen) { _spliceMap = Map.fromList $ splicesToList splices
+preproc keyGen splices templates ns = do
+    sm <- runHashMap splices
+    let hs = (emptyHS keyGen) { _spliceMap = sm
                               , _templateMap = templates
-                              , _preprocessingMode = True }
+                              , _preprocessingMode = True
+                              , _splicePrefix = mkSplicePrefix ns }
     let eval a = evalHeistT a (X.TextNode "") hs
     tPairs <- lift $ mapM (eval . preprocess) $ Map.toList templates
     let bad = lefts tPairs
@@ -296,21 +311,21 @@ preprocess (tpath, docFile) = do
 initHeistWithCacheTag :: MonadIO n
                       => HeistConfig n
                       -> EitherT [String] IO (HeistState n, CacheTagState)
-initHeistWithCacheTag (HeistConfig i lt c a locations) = do
+initHeistWithCacheTag (HeistConfig sc ns enn) = do
     (ss, cts) <- liftIO mkCacheTag
     let tag = "cache"
     keyGen <- lift HE.newKeyGen
 
-    repos <- sequence locations
+    repos <- sequence $ _scTemplateLocations sc
     -- We have to do one preprocessing pass with the cache setup splice.  This
     -- has to happen for both interpreted and compiled templates, so we do it
     -- here by itself because interpreted templates don't get the same load
     -- time splices as compiled templates.
-    rawWithCache <- preproc keyGen (tag ## ss) $ Map.unions repos
+    rawWithCache <- preproc keyGen (tag ## ss) (Map.unions repos) ns
 
-    let hc' = HeistConfig (insertS tag (cacheImpl cts) i) lt
-                          (insertS tag (cacheImplCompiled cts) c)
-                          a locations
-    hs <- initHeist' keyGen hc' rawWithCache
+    let sc' = SpliceConfig (tag #! cacheImpl cts) mempty
+                           (tag #! cacheImplCompiled cts) mempty mempty
+    let hc = HeistConfig (mappend sc sc') ns enn
+    hs <- initHeist' keyGen hc rawWithCache
     return (hs, cts)
 
