@@ -4,25 +4,47 @@
 
 module Heist.Common where
 
-import           Control.Applicative
-import           Control.Exception (SomeException)
-import           Control.Monad
-import qualified Control.Monad.CatchIO as C
-import qualified Data.Attoparsec.Text            as AP
-import           Data.ByteString (ByteString)
-import qualified Data.ByteString as B
-import qualified Data.ByteString.Char8 as BC
-import           Data.Hashable
-import           Data.HashMap.Strict (HashMap)
-import qualified Data.HashMap.Strict as Map
-import           Data.List
-import           Data.Maybe
-import           Data.Monoid
-import qualified Data.Text                       as T
-import           System.FilePath
-import           Heist.SpliceAPI
-import           Heist.Types
-import qualified Text.XmlHtml as X
+------------------------------------------------------------------------------
+import           Control.Applicative      (Alternative (..), Applicative (..), (<$>))
+import           Control.Error
+import           Control.Exception        (SomeException)
+import qualified Control.Monad.CatchIO    as C
+import           Control.Monad            (liftM, mplus)
+import qualified Data.Attoparsec.Text     as AP
+import           Data.ByteString          (ByteString)
+import qualified Data.ByteString          as B
+import qualified Data.ByteString.Char8    as BC
+import           Data.Hashable            (Hashable)
+import           Data.HashMap.Strict      (HashMap)
+import qualified Data.HashMap.Strict      as Map
+import           Data.List                (isSuffixOf)
+import           Data.Map.Syntax
+import           Data.Monoid              (Monoid (..), (<>))
+import           Data.Text                (Text)
+import qualified Data.Text                as T
+import           Heist.Internal.Types.HeistState
+import           System.FilePath          (pathSeparator)
+import qualified Text.XmlHtml             as X
+------------------------------------------------------------------------------
+
+
+------------------------------------------------------------------------------
+runHashMap
+    :: (Monad m)
+    => Splices s
+    -> EitherT [String] m (HashMap T.Text s)
+runHashMap ms =
+    case runMapSyntax Map.lookup Map.insert ms of
+      Left keys -> left $ map (T.unpack . mkMsg) keys
+      Right hm -> right $ hm
+  where
+    mkMsg k = "You tried to bind "<>k<>" more than once!"
+
+
+------------------------------------------------------------------------------
+runMapNoErrors :: (Eq k, Hashable k) => MapSyntaxM k v a -> HashMap k v
+runMapNoErrors = either (const mempty) id .
+    runMapSyntax' (\_ new _ -> Just new) Map.lookup Map.insert
 
 
 ------------------------------------------------------------------------------
@@ -37,15 +59,36 @@ orError :: Monad m => HeistT n m b -> String -> HeistT n m b
 orError silent msg = do
     hs <- getHS
     if _preprocessingMode hs
-      then error $ (maybe "" (++": ") $ _curTemplateFile hs) ++ msg
+      then do fullMsg <- heistErrMsg (T.pack msg)
+              error $ T.unpack fullMsg
       else silent
 
 
+------------------------------------------------------------------------------
+-- | Prepends the location of the template currently being processed to an
+-- error message.
+heistErrMsg :: Monad m => Text -> HeistT n m Text
+heistErrMsg msg = do
+    tf <- getsHS _curTemplateFile
+    return $ (maybe "" ((`mappend` ": ") . T.pack) tf) `mappend` msg
+
+
+------------------------------------------------------------------------------
+-- | Adds an error message to the list of splice processing errors.
+tellSpliceError :: Monad m => Text -> HeistT n m ()
+tellSpliceError msg = do
+    fullMsg <- heistErrMsg msg
+    modifyHS (\hs -> hs { _spliceErrors = fullMsg : _spliceErrors hs })
+
+
 ------------------------------------------------------------------------------
 -- | Function for showing a TPath.
 showTPath :: TPath -> String
 showTPath = BC.unpack . (`BC.append` ".tpl") . tpathName
 
+
+------------------------------------------------------------------------------
+-- | Convert a TPath into a ByteString path.
 tpathName :: TPath -> ByteString
 tpathName = BC.intercalate "/" . reverse
 
@@ -294,7 +337,7 @@ bindAttributeSplices :: Splices (AttrSplice n) -- ^ splices to bind
                      -> HeistState n           -- ^ start state
                      -> HeistState n
 bindAttributeSplices ss hs =
-    hs { _attrSpliceMap = Map.union (Map.fromList $ splicesToList ss)
+    hs { _attrSpliceMap = Map.union (runMapNoErrors ss)
                                     (_attrSpliceMap hs) }
 
 
